<?php

namespace Stachethemes\Stec;

class GDPR {

    private static $cache_string_prefix = 'stec_gdpr_cache_';

    // In stachethemes_event_calendar\inc\inc.task-handler.php
    public static function task_handler() {

        // Attendance by URL Listener
        $give_consent = Admin_Helper::isset_get('stec_gdpr_give_consent');
        $token        = Admin_Helper::get('token', null, FILTER_DEFAULT);
        $stec         = Stachethemes_Event_Calendar::get_instance();

        if ($give_consent) {

            try {

                $token_data = Admin_Helper::get_cache(self::$cache_string_prefix . $token, true);

                if (!trim($token_data)) {
                    throw new Stec_Exception('GDPR::task_handler No token data');
                }

                $extracted_data = self::exctract_encrypted_data($token_data);
                $email          = filter_var($extracted_data['email'], FILTER_VALIDATE_EMAIL);

                if (!$email) {
                    throw new Stec_Exception('GDPR::task_handler No email found in token data');
                }

                if (self::is_confirmed($email)) {
                    throw new Stec_Exception('GDPR::task_handler Email already confirmed');
                }
                
            } catch (Stec_Exception $ex) {
                Admin_Helper::debug_log($ex->getMessage());
                wp_redirect(site_url('/'));
                exit;
            }

            self::add_email($email);
            self::process_data($extracted_data);
            Mail::send_mail_confirmed($email);

            $stec->set_fixed_message(esc_html__('Your email has been confirmed', 'stec'), 'success');
            wp_redirect(site_url('/'));
            exit;
        }
    }

    private static function exctract_encrypted_data($data) {
        try {
            $decode_data = Admin_Helper::decrypt($data);
            parse_str($decode_data, $parsed_data);

            return $parsed_data;
        } catch (Stec_Exception $ex) {
            Admin_Helper::debug_log('GDPR exctract_encrypted_data:: ' . $ex->getMessage());
            return false;
        }
    }

    private static function process_data($data) {

        try {
            switch ($data['task']) {

                case 'gdpr_rsvp_to_event' :

                    $event_id = $data['event_id'];
                    $date     = $data['date'];
                    $attendee = unserialize($data['attendee']);
                    $event    = new Event_Post($event_id);

                    try {
                        Rsvp::add($event, $date, $attendee);
                    } catch (Stec_Exception $ex) {
                        return false;
                    }

                    break;


                case 'gdpr_set_reminder' :

                    $event_id      = $data['event_id'];
                    $repeat_offset = $data['repeat_offset'];
                    $date          = $data['date'];
                    $email         = $data['email'];

                    Reminder::set($event_id, $repeat_offset, $email, $date);

                    break;

                case 'gdpr_send_event_submitted_owner_mail' :

                    $event = new Event_Post($data['event_id']);
                    Mail::send_event_submitted_owner_mail($event);

                    break;
            }
        } catch (Stec_Exception $ex) {
            return false;
        }

        return true;
    }

    private static function add_email($email) {

        try {
            $email = Admin_Helper::encrypt($email);
        } catch (Stec_Exception $ex) {
            Admin_Helper::debug_log($ex->getMessage());
        }

        $emails = self::get_emails();
        array_push($emails, $email);
        update_option('stec_gdpr_emails', $emails, false);
    }

    public static function get_emails() {
        $emails = get_option('stec_gdpr_emails', array());

        foreach ($emails as &$email) {
            if (false === filter_var($email, FILTER_VALIDATE_EMAIL)) {
                try {
                    $email = Admin_Helper::decrypt($email);
                } catch (Stec_Exception $ex) {
                    Admin_Helper::debug_log('GDPR get_emails:: ' . $ex->getMessage());
                }
            }
        }

        return $emails;
    }

    public static function is_confirmed($email) {

        // Check if GDPR is disabled
        if (0 === (int) Settings::get_admin_setting_value('stec_menu__general_privacy_agreement', 'gdpr_enabled')) {
            return true;
        }

        $emails = self::get_emails();
        return in_array($email, $emails);
    }

    public static function request_confirmation($to, $data) {

        $token      = wp_hash(uniqid(time()));
        $token_data = Admin_Helper::encrypt($data);

        Admin_Helper::set_cache(self::$cache_string_prefix . $token, $token_data, 604800, true);

        $confirmation_link = site_url() . "?stec_gdpr_give_consent&token=$token";

        return Mail::send_gdpr_request_confirmation($to, $confirmation_link);
    }

    public static function delete_emails() {
        delete_option('stec_gdpr_emails');
    }

}
