<?php

namespace Stachethemes\Stec;

class Mail {

    private static function get_author_email(Event_Post $event) {

        try {
            $contact_email = $event->get_custom_meta('contact_email');

            if (false === filter_var($contact_email, FILTER_VALIDATE_EMAIL)) {

                if (!$event->get_author() || $event->get_author() <= 0) {
                    throw new Stec_Exception('get_author_email:: Invalid author id: ' . $event->get_author());
                }

                $userdata = get_userdata($event->get_author());

                if (!$userdata) {
                    throw new Stec_Exception('get_author_email:: Invalid author e-mail: ' . $contact_email);
                }

                if (!isset($userdata->user_email) || false === filter_var($userdata->user_email, FILTER_VALIDATE_EMAIL)) {
                    throw new Stec_Exception('get_author_email:: Invalid author e-mail: ' . $contact_email);
                }

                $contact_email = $userdata->user_email;
            }
        } catch (Stec_Exception $ex) {
            Admin_Helper::debug_log($ex->getMessage());
            return false;
        }

        return $contact_email;
    }

    private static function add_special_words($text, Event_Post $event = null, Array $add = array()) {

        $filter = array(
            'replace' => array(
                '{{title}}',
                '{{permalink}}',
                '{{start_date}}'
            ),
            'with'    => array(
                $event ? $event->get_title() : '',
                $event ? $event->get_permalink() : '',
                $event ? $event->get_start_date('Y-m-d H:i') : ''
            )
        );

        if ($add) {
            $filter['replace'] = array_merge($filter['replace'], $add['replace']);
            $filter['with']    = array_merge($filter['with'], $add['with']);
        }

        $filter = apply_filters('stec_mail_add_special_words', $filter, $text, $event);

        return str_replace($filter['replace'], $filter['with'], $text);
    }

    /**
     * Notify organizers and attendees the event has been changed
     */
    public static function send_event_changed_mail(Event_Post $old_event, Event_Post $new_event) {

        if (1 !== (int) Settings::get_admin_setting_value('stec_menu__general_email_notifications', 'send_event_changed_mail')) {
            return;
        }

        $attendees  = $old_event->get_attendance(true);
        $organizers = $old_event->get_organizers();

        $email_list = array();

        foreach ($attendees as $attendee) {

            if (false === ($attendee instanceof Event_Meta_Attendee)) {
                continue;
            }

            if ($attendee->get_email()) {
                $email_list[] = $attendee->get_email();
            }
        }

        foreach ($organizers as $organizer) {

            if (false === ($attendee instanceof Organizer)) {
                continue;
            }

            if ($organizer->get_email()) {
                $email_list[] = $organizer->get_email();
            }
        }

        $subject = Settings::get_admin_setting_value('stec_menu__general_email_templates', 'send_event_changed_mail_subject', true);
        $content = Settings::get_admin_setting_value('stec_menu__general_email_templates', 'send_event_changed_mail_content', true);

        $filter = apply_filters('stec_send_event_changed_mail', array(
            'emails'  => $email_list,
            'subject' => $subject,
            'content' => $content
                ), $old_event, $new_event);

        $filter['subject'] = self::add_special_words($filter['subject'], $new_event);
        $filter['content'] = self::add_special_words($filter['content'], $new_event);

        if (isset($filter['emails']) && is_array($filter['emails']) && $filter['emails']) {
            foreach ($filter['emails'] as $email) {

                if (false === filter_var($email, FILTER_VALIDATE_EMAIL)) {
                    Admin_Helper::debug_log('send_event_changed_mail:: Invalid email: ' . $email);
                    continue;
                }

                if (false === wp_mail($email, $filter['subject'], $filter['content'])) {
                    Admin_Helper::debug_log('send_event_changed_mail:: Unable to send email to: ' . $email);
                }
            }
        }
    }

    /**
     * Notify event owner the event has been rejected
     */
    public static function send_event_rejected_mail(Event_Post $event) {

        if (1 !== (int) Settings::get_admin_setting_value('stec_menu__general_email_notifications', 'send_event_rejected_mail')) {
            return;
        }

        try {
            $contact_email = self::get_author_email($event);
        } catch (Stec_Exception $ex) {
            Admin_Helper::debug_log('send_event_rejected_mail:: ' . $ex->getMessage());
            return false;
        }

        $subject = Settings::get_admin_setting_value('stec_menu__general_email_templates', 'send_event_rejected_mail_subject', true);
        $content = Settings::get_admin_setting_value('stec_menu__general_email_templates', 'send_event_rejected_mail_content', true);

        $filter = apply_filters('stec_send_event_rejected_mail', array(
            'subject' => $subject,
            'content' => $content
                ), $event);

        $filter['subject'] = self::add_special_words($filter['subject'], $event);
        $filter['content'] = self::add_special_words($filter['content'], $event);

        if (GDPR::is_confirmed($contact_email)) {
            if (false === wp_mail($contact_email, $filter['subject'], $filter['content'])) {
                Admin_Helper::debug_log('send_event_rejected_mail:: Unable to send email to: ' . $contact_email);
            }
        }
    }

    /**
     * Send invitations to event attendees
     */
    public static function send_event_invitations_mail(Event_Post $event) {

        if (1 !== (int) Settings::get_admin_setting_value('stec_menu__general_email_notifications', 'send_event_invitations_mail')) {
            return;
        }

        $attendees = $event->get_attendance(true);

        if (!$attendees) {
            return;
        }

        $site_url = site_url();

        $subject = Settings::get_admin_setting_value('stec_menu__general_email_templates', 'send_event_invitations_mail_subject', true);
        $content = Settings::get_admin_setting_value('stec_menu__general_email_templates', 'send_event_invitations_mail_content', true);

        $filter = apply_filters('stec_send_event_invitations_mail', array(
            'subject' => $subject,
            'content' => $content
                ), $event);

        foreach ($attendees as &$attendee) {

            if (!$attendee instanceof Event_Meta_Attendee) {
                continue;
            }

            $email = filter_var($attendee->get_email(), FILTER_VALIDATE_EMAIL);

            if ($attendee->get_mail_sent() === 1) {
                continue;
            }

            if (!$email) {

                if ($attendee->get_userid()) {
                    $userdata = get_userdata($attendee->get_userid());
                    $email    = $userdata->user_email;
                }
            }

            if (!$email) {
                continue;
            }

            $accept_url  = $site_url . "?stec_attendance={$event->get_id()}&access_token={$attendee->get_access_token()}&status=1";
            $decline_url = $site_url . "?stec_attendance={$event->get_id()}&access_token={$attendee->get_access_token()}&status=2";

            $filter['subject'] = self::add_special_words($filter['subject'], $event, array(
                        'replace' => array('{{accept_link}}', '{{decline_link}}'),
                        'with'    => array($accept_url, $decline_url)
            ));

            $filter['content'] = self::add_special_words($filter['content'], $event, array(
                        'replace' => array('{{accept_link}}', '{{decline_link}}'),
                        'with'    => array($accept_url, $decline_url)
            ));

            if (false !== wp_mail($email, $filter['subject'], $filter['content'])) {
                $attendee->set_mail_sent(1);
            } {
                Admin_Helper::debug_log('send_event_invitations_mail:: Unable to send email to: ' . $email);
            }
        }

        update_post_meta($event->get_id(), 'attendance', '');
        update_post_meta($event->get_id(), 'attendance', Admin_Helper::encrypt($attendees));
    }

    /**
     * Notify event owner the event has been approved
     */
    public static function send_event_approved_mail(Event_Post $event) {

        if (1 !== (int) Settings::get_admin_setting_value('stec_menu__general_email_notifications', 'send_event_approved_mail')) {
            return;
        }

        try {
            $contact_email = self::get_author_email($event);
        } catch (Stec_Exception $ex) {
            Admin_Helper::debug_log('send_event_approved_mail:: ' . $ex->getMessage());
            return false;
        }

        $subject = Settings::get_admin_setting_value('stec_menu__general_email_templates', 'send_event_approved_mail_subject', true);
        $content = Settings::get_admin_setting_value('stec_menu__general_email_templates', 'send_event_approved_mail_content', true);

        $filter = apply_filters('stec_send_event_approved_mail', array(
            'subject' => $subject,
            'content' => $content
                ), $event);

        $filter['subject'] = self::add_special_words($filter['subject'], $event);
        $filter['content'] = self::add_special_words($filter['content'], $event);

        if (GDPR::is_confirmed($contact_email)) {
            if (false === wp_mail($contact_email, $filter['subject'], $filter['content'])) {
                Admin_Helper::debug_log('send_event_approved_mail:: Unable to send e-mail to: ' . $contact_email);
            }
        }
    }

    /**
     * Notify admin an event is awaiting approval via mail
     */
    public static function send_event_submitted_admin_mail(Event_Post $event) {

        if (1 !== (int) Settings::get_admin_setting_value('stec_menu__general_email_notifications', 'send_event_submitted_admin_mail')) {
            return;
        }

        $user_info = get_user_by('id', $event->get_author());
        $author    = isset($user_info->display_name) ? $user_info->display_name : 'Anonymous';
        $admin_url = esc_url(admin_url('admin.php?page=stec_menu__events&view=edit&calendar_id=' . $event->get_calid() . '&event_id=' . $event->get_id()));

        $subject = Settings::get_admin_setting_value('stec_menu__general_email_templates', 'send_event_submitted_admin_mail_subject', true);
        $content = Settings::get_admin_setting_value('stec_menu__general_email_templates', 'send_event_submitted_admin_mail_content', true);

        $filter = apply_filters('stec_send_event_submitted_admin_mail', array(
            'to'      => get_option('admin_email'),
            'subject' => $subject,
            'content' => $content
                ), $event);

        $filter['subject'] = self::add_special_words($filter['subject'], $event, array(
                    'replace' => array('{{admin_permalink}}', '{{author}}'),
                    'with'    => array($admin_url, $author)
        ));

        $filter['content'] = self::add_special_words($filter['content'], $event, array(
                    'replace' => array('{{admin_permalink}}', '{{author}}'),
                    'with'    => array($admin_url, $author)
        ));

        if (false === wp_mail($filter['to'], $filter['subject'], $filter['content'])) {
            Admin_Helper::debug_log('send_event_submitted_admin_mail:: Unable to send e-mail to: ' . $filter['to']);
        }
    }

    /**
     * Notify owner their event is submitted for approval via mail
     */
    public static function send_event_submitted_owner_mail(Event_Post $event) {

        if (1 !== (int) Settings::get_admin_setting_value('stec_menu__general_email_notifications', 'send_event_submitted_owner_mail')) {
            return;
        }

        try {
            $to = self::get_author_email($event);
        } catch (Stec_Exception $ex) {
            Admin_Helper::debug_log('send_event_submitted_owner_mail:: ' . $ex->getMessage());
            return false;
        }

        $subject = Settings::get_admin_setting_value('stec_menu__general_email_templates', 'send_event_submitted_owner_mail_subject', true);
        $content = Settings::get_admin_setting_value('stec_menu__general_email_templates', 'send_event_submitted_owner_mail_content', true);

        $filter = apply_filters('stec_send_event_submitted_owner_mail', array(
            'subject' => $subject,
            'content' => $content
                ), $event);

        $filter['subject'] = self::add_special_words($filter['subject'], $event);
        $filter['content'] = self::add_special_words($filter['content'], $event);

        if (GDPR::is_confirmed($to)) {
            if (false === wp_mail($to, $filter['subject'], $filter['content'])) {
                Admin_Helper::debug_log('send_event_submitted_owner_mail:: Unable to send e-mail to: ' . $to);
            }
        } else {

            $data_query = http_build_query(array(
                'task'     => 'gdpr_event_submit_notify_owner',
                'event_id' => $event->get_id(),
                'email'    => $to
            ));

            GDPR::request_confirmation($to, $data_query);
        }
    }

    /**
     * Send reminds
     * @param \Stachethemes\Stec\Remind_Object $remind
     * @return bool true if email is sent or remind is invalid or event doesn't exist
     */
    public static function send_event_remind_mail(Remind_Object $remind) {

        if (1 !== (int) Settings::get_admin_setting_value('stec_menu__general_email_notifications', 'send_event_remind_mail')) {
            return;
        }

        $data = $remind->get_data();

        if (!isset($data['eventid'])) {
            return true; // consider task complete
        }

        $event = new Event_Post($data['eventid']);

        if (!$event->get_id()) {
        // Event does not exists consider task complete
            return true;
        }

        $email = $data['email'];

        if (false === filter_var($email, FILTER_VALIDATE_EMAIL)) {
            Admin_Helper::debug_log('send_event_remind_mail:: Invalid email: ' . $email);
        // Invalid e-mail consider task complete
            return true;
        }

        $gm_now         = gmdate(time());
        $gm_remind_date = strtotime(Admin_Helper::to_utc_time($data['date'], $event->get_timezone()));

        if ($gm_now < $gm_remind_date) {
            // Not yet...
            return false;
        }

        $subject = Settings::get_admin_setting_value('stec_menu__general_email_templates', 'send_event_remind_mail_subject', true);
        $content = Settings::get_admin_setting_value('stec_menu__general_email_templates', 'send_event_remind_mail_content', true);

        $filter = apply_filters('stec_send_event_remind_mail', array(
            'subject' => $subject,
            'content' => $content
                ), $remind);

        /**
         * @todo
         * Repeat offset start date?
         */
        $filter['subject'] = self::add_special_words($filter['subject'], $event);
        $filter['content'] = self::add_special_words($filter['content'], $event);

        Admin_Helper::debug_log('Remind job sending email to ' . $email);

        if (false !== wp_mail($email, $filter['subject'], $filter['content'])) {
            return true;
        }

        Admin_Helper::debug_log('send_event_remind_mail:: Unable to send email to: ' . $email);
        return false;
    }

    /**
     * Request GDPR consent
     */
    public static function send_gdpr_request_confirmation($email, $confirmation_link) {

        if (1 !== (int) Settings::get_admin_setting_value('stec_menu__general_email_notifications', 'send_gdpr_request_confirmation')) {
            return;
        }

        if (false === filter_var($email, FILTER_VALIDATE_EMAIL)) {
            Admin_Helper::debug_log('send_gdpr_request_confirmation:: Invalid email: ' . $email);
            return false;
        }

        $subject = Settings::get_admin_setting_value('stec_menu__general_email_templates', 'send_gdpr_request_confirmation_subject', true);
        $content = Settings::get_admin_setting_value('stec_menu__general_email_templates', 'send_gdpr_request_confirmation_content', true);

        $filter = apply_filters('stec_send_gdpr_request_confirmation', array(
            'subject' => $subject,
            'content' => $content
                ), $confirmation_link);

        $filter['subject'] = self::add_special_words($filter['subject'], null, array(
                    'replace' => array('{{gdpr_link}}'),
                    'with'    => array($confirmation_link)
        ));

        $filter['content'] = self::add_special_words($filter['content'], null, array(
                    'replace' => array('{{gdpr_link}}'),
                    'with'    => array($confirmation_link)
        ));
        
        $result = wp_mail($email, $filter['subject'], $filter['content']);

        if (false === $result) {
            Admin_Helper::debug_log('send_gdpr_request_confirmation:: Unable to send email to: ' . $email);
        }

        return $result;
    }

    /**
     * GDPR Mail confirmed
     */
    public static function send_mail_confirmed($email) {

        if (1 !== (int) Settings::get_admin_setting_value('stec_menu__general_email_notifications', 'send_mail_confirmed')) {
            return;
        }

        $subject = Settings::get_admin_setting_value('stec_menu__general_email_templates', 'send_mail_confirmed_subject', true);
        $content = Settings::get_admin_setting_value('stec_menu__general_email_templates', 'send_mail_confirmed_content', true);

        $filter = apply_filters('stec_send_mail_confirmed', array(
            'subject' => $subject,
            'content' => $content
        ));

        if (GDPR::is_confirmed($email)) {
            if (false === wp_mail($email, $filter['subject'], $filter['content'])) {
                Admin_Helper::debug_log('send_mail_confirmed:: Unable to send email to: ' . $email);
            }
        }
    }

    /**
     * RSVP Status accepted
     */
    public static function send_event_rsvp_accepted($email, Event_Post $event) {

        if (1 !== (int) Settings::get_admin_setting_value('stec_menu__general_email_notifications', 'send_event_rsvp_accepted')) {
            return;
        }

        $subject = Settings::get_admin_setting_value('stec_menu__general_email_templates', 'send_event_rsvp_accepted_subject', true);
        $content = Settings::get_admin_setting_value('stec_menu__general_email_templates', 'send_event_rsvp_accepted_content', true);

        $filter = apply_filters('stec_send_event_rsvp_accepted', array(
            'subject' => $subject,
            'content' => $content
                ), $event);

        $filter['subject'] = self::add_special_words($filter['subject'], $event);
        $filter['content'] = self::add_special_words($filter['content'], $event);

        if (GDPR::is_confirmed($email)) {
            if (false === wp_mail($email, $filter['subject'], $filter['content'])) {
                Admin_Helper::debug_log('send_event_rsvp_accepted:: Unable to send email to: ' . $email);
            }
        }
    }

    /**
     * Send event canceled
     */
    public static function send_event_cancelled_mail(Event_Post $event) {

        if (1 !== (int) Settings::get_admin_setting_value('stec_menu__general_email_notifications', 'send_event_changed_mail')) {
            return;
        }

        $subject = Settings::get_admin_setting_value('stec_menu__general_email_templates', 'send_event_cancelled_mail_subject', true);
        $content = Settings::get_admin_setting_value('stec_menu__general_email_templates', 'send_event_cancelled_mail_content', true);

        $filter = apply_filters('stec_send_event_cancelled_mail', array(
            'subject' => $subject,
            'content' => $content
                ), $event);

        $filter['subject'] = self::add_special_words($filter['subject'], $event);
        $filter['content'] = self::add_special_words($filter['content'], $event);

        $attendees = $event->get_attendance(true);

        if ($attendees) {
            foreach ($attendees as $attendee) {

                if (!$attendee instanceof Event_Meta_Attendee) {
                    continue;
                }

                $email = filter_var($attendee->get_email(), FILTER_VALIDATE_EMAIL);

                if (!$email) {
                    if ($attendee->userid) {
                        $userdata = get_userdata($attendee->userid);
                        $email    = $userdata->user_email;
                    }
                }

                if (!$email) {
                    continue;
                }

                if (false === wp_mail($email, $filter['subject'], $filter['content'])) {
                    Admin_Helper::debug_log('send_event_cancelled_mail:: Unable to send email to: ' . $email);
                }
            }
        }
    }

    /**
     * Notify event author new attendee has been added via rsvp function
     */
    public static function send_event_author_new_attendee(Event_Meta_Attendee $attendee, Event_Post $event, $date) {

        if (1 !== (int) Settings::get_admin_setting_value('stec_menu__general_email_notifications', 'send_event_author_new_attendee')) {
            return;
        }

        try {
            $to = self::get_author_email($event);
        } catch (Stec_Exception $ex) {
            Admin_Helper::debug_log('send_event_author_new_attendee:: ' . $ex->getMessage());
            return false;
        }

        $subject = Settings::get_admin_setting_value('stec_menu__general_email_templates', 'send_event_author_new_attendee_subject', true);
        $content = Settings::get_admin_setting_value('stec_menu__general_email_templates', 'send_event_author_new_attendee_content', true);

        $filter = apply_filters('stec_send_event_author_new_attendee', array(
            'subject' => $subject,
            'content' => $content
                ), $event, $attendee, $date);

        $filter['subject'] = self::add_special_words($filter['subject'], $event, array(
                    'replace' => array('{{attendee}}', '{{attend_date}}'),
                    'with'    => array($attendee->get_name(), $date)
        ));

        $filter['content'] = self::add_special_words($filter['content'], $event, array(
                    'replace' => array('{{attendee}}', '{{attend_date}}'),
                    'with'    => array($attendee->get_name(), $date)
        ));

        if (false === wp_mail($to, $filter['subject'], $filter['content'])) {
            Admin_Helper::debug_log('send_event_author_new_attendee:: Unable to send email to: ' . $to);
        }
    }

}