<?php

namespace Stachethemes\Stec;

/**
 * Front Create/Delete event
 */
class Submit_Event {

    public static function get_user_cookie($create = true) {

        $cookie_name = "stachethemes_ec_anon_user";

        if (!isset($_COOKIE[$cookie_name])) {

            if ($create === true) {

                $id = uniqid('stec-user-');

                if (setcookie($cookie_name, $id, time() + (10 * 365 * 24 * 60 * 60), "/") === false) {
                    return null;
                }

                $_COOKIE[$cookie_name] = $id;
            }
        }

        return isset($_COOKIE[$cookie_name]) && strlen($_COOKIE[$cookie_name]) > 13 ? $_COOKIE[$cookie_name] : null;
    }

    /**
     * Check must have fields for event to be registered without errors
     * Must have title, calid, start and end date
     * @throws Stec_Exception
     */
    private static function validate_must_fields() {

        if (!sanitize_text_field(Admin_Helper::post('title', false, FILTER_SANITIZE_STRING))) {
            throw new Stec_Exception(esc_html__('Event must have title', 'stec'));
        }

        if (!Admin_Helper::post('calendar_id', false, FILTER_VALIDATE_INT)) {
            throw new Stec_Exception(esc_html__('You must select calendar', 'stec'));
        }

        if (!Admin_Helper::post('start_date', false, FILTER_SANITIZE_STRING)) {
            throw new Stec_Exception(esc_html__('Event must have start date', 'stec'));
        }

        if (!Admin_Helper::post('end_date', false, FILTER_SANITIZE_STRING)) {
            throw new Stec_Exception(esc_html__('Event must have end date', 'stec'));
        }
    }

    /**
     * Creates event
     * @return event|false
     */
    public static function create_event() {

        self::validate_must_fields();

        $event_id  = Admin_Helper::post('event_id', null, FILTER_VALIDATE_INT);
        $calendar  = new Calendar_Post(Admin_Helper::post('calendar_id', false, FILTER_VALIDATE_INT));
        $event     = new Event_Post($event_id);
        $old_event = null;
        $is_edit   = false;

        if ($event->get_id()) {
            $old_event = clone $event;
            $is_edit   = true;
        }

        if (!Calendars::user_can_write_calendar($calendar)) {
            throw new Stec_Exception(esc_html__("You don't have permission to submit events to this calendar", 'stec'));
        }

        if (true === $is_edit) {
            if (false === Events::user_can_edit_event($event)) {
                // if anon user is not get user cookie then false
                throw new Stec_Exception(esc_html__("You don't have permission to edit this event", 'stec'));
            }
        }

        $event->set_calid(Admin_Helper::post('calendar_id', false, FILTER_VALIDATE_INT));
        $event->set_title(Admin_Helper::post('title', esc_html__('Untitled', 'stec'), FILTER_SANITIZE_STRING));

        if (Admin_Helper::isset_post('event_color')) {
            $event->set_color(Admin_Helper::post('event_color', '#f15e6e'));
        }

        if (Admin_Helper::isset_post('icon')) {
            $event->set_icon(Admin_Helper::post('icon', 'fa'));
        }

        if (Admin_Helper::isset_post('featured')) {
            $event->set_featured(Admin_Helper::post('featured', 0, FILTER_VALIDATE_INT));
        }

        if (Admin_Helper::isset_post('event_categories')) {
            $event->set_categories(Admin_Helper::post('event_categories', array(), FILTER_VALIDATE_INT, FILTER_REQUIRE_ARRAY));
        }

        $start_date         = Admin_Helper::post('start_date_alt');
        $start_time_hours   = Admin_Helper::post('start_time_hours', '00');
        $start_time_minutes = Admin_Helper::post('start_time_minutes', '00');
        $end_date           = Admin_Helper::post('end_date_alt');
        $end_time_hours     = Admin_Helper::post('end_time_hours', '00');
        $end_time_minutes   = Admin_Helper::post('end_time_minutes', '00');
        $start_date_time    = $start_date . ' ' . $start_time_hours . ':' . $start_time_minutes . ':00';
        $end_date_time      = $end_date . ' ' . $end_time_hours . ':' . $end_time_minutes . ':00';
        $event->set_start_date($start_date_time);
        $event->set_end_date($end_date_time);
        $event->set_all_day(Admin_Helper::post('all_day', false) !== false ? 1 : 0);

        if (Admin_Helper::isset_post('rrule')) {
            $event->set_rrule(Admin_Helper::post('rrule', ''));
            $event->set_exdate(Admin_Helper::post('exdate', ''));
            $event->set_is_advanced_rrule(Admin_Helper::post('is_advanced_rrule', 0, FILTER_VALIDATE_INT));
        }

        if (Admin_Helper::isset_post('keywords')) {
            $event->set_keywords(Admin_Helper::post('keywords', '', FILTER_SANITIZE_STRING));
        }

        if (Admin_Helper::isset_post('counter')) {
            $event->set_counter(Admin_Helper::post('counter', 0, FILTER_VALIDATE_INT));
        }

        if (Admin_Helper::isset_post('comments')) {
            $event->set_comments(Admin_Helper::post('comments', 0, FILTER_VALIDATE_INT));
        }

        if (Admin_Helper::isset_post('rsvp')) {
            $rsvp_settings = Admin_Helper::post('rsvp', array(), FILTER_DEFAULT, FILTER_REQUIRE_ARRAY);

            if (isset($rsvp_settings['collect']) && false !== filter_var($rsvp_settings['collect'], FILTER_VALIDATE_INT)) {
                $event->set_rsvp($rsvp_settings['collect']);
            }

            if (isset($rsvp_settings['limit']) && false !== filter_var($rsvp_settings['limit'], FILTER_VALIDATE_INT)) {
                $event->set_rsvp_limit($rsvp_settings['limit']);
            }

            if (isset($rsvp_settings['visibility'])) {
                $event->set_attendance_visibility($rsvp_settings['visibility']);
            }
        }

        if (Admin_Helper::isset_post('link')) {
            $event->set_link(Admin_Helper::post('link', '', FILTER_VALIDATE_URL));
        }

        if (isset($_FILES['stec_public_file_image'])) {
            $image_id = Admin_Helper::file_image_to_media_upload($_FILES['stec_public_file_image']['tmp_name'], $_FILES['stec_public_file_image']['name'], false);
            if (filter_var($image_id, FILTER_VALIDATE_INT)) {
                $event->set_images(array($image_id));
            }
        } elseif (is_user_logged_in() && Admin_Helper::isset_post('stec_has_media')) {
            $event->set_images(Admin_Helper::post('images', array(), FILTER_VALIDATE_INT, FILTER_REQUIRE_ARRAY));
        }

        if (isset($_FILES['stec_public_file_attachment'])) {
            $file_id = Admin_Helper::file_to_media_upload($_FILES['stec_public_file_attachment']['tmp_name'], $_FILES['stec_public_file_attachment']['name'], false);
            if (filter_var($file_id, FILTER_VALIDATE_INT)) {
                $event->set_attachments(array());
                $attachment = new Event_Meta_Attachment();
                $attachment->set_id($file_id);
                $event->set_attachment($attachment);
            }
        } elseif (is_user_logged_in() && Admin_Helper::isset_post('has_attachments')) {
            
            $attachments = Admin_Helper::post('attachments', array(), FILTER_VALIDATE_INT, FILTER_REQUIRE_ARRAY);
            $event->set_attachments(array());

            foreach ($attachments as $attachment_id) {
                $attachment = new Event_Meta_Attachment();
                $attachment->set_id($attachment_id);
                $event->set_attachment($attachment);
            }
        }

        if (Admin_Helper::isset_post('description')) {
            $event->set_description(Admin_Helper::post('description', ''));
        }

        if (Admin_Helper::isset_post('description_short')) {
            $event->set_description_short(Admin_Helper::post('description_short', ''));
        }

        if (Admin_Helper::isset_post('location')) {
            $location = Admin_Helper::post('location');

            switch ($location) {
                case 'custom_address':
                case 'custom' :
                    if (Admin_Helper::isset_post('location_address')) {
                        $location_address = Admin_Helper::post('location_address', '', FILTER_SANITIZE_STRING);
                        $location_city    = Admin_Helper::post('location_city', '', FILTER_SANITIZE_STRING);
                        $location_country = Admin_Helper::post('location_country', '', FILTER_SANITIZE_STRING);
                        $location_coords  = Admin_Helper::post('location_coordinates', '', FILTER_DEFAULT);
                        $location_details = Admin_Helper::post('location_details', '', FILTER_DEFAULT);

                        $create_event_form_allow_location_type = (int) Settings::get_admin_setting_value('stec_menu__general_event_submit', 'create_event_form_allow_location_type');

                        if (0 === $create_event_form_allow_location_type) {
                            throw new Stec_Exception(esc_html__('You do not have permission to add custom locations', 'stec'));
                        }

                        if (1 === $create_event_form_allow_location_type) {
                            if (false === Locations::has_country($location_country)) {
                                throw new Stec_Exception(esc_html__('Invalid location country', 'stec'));
                            }
                            if (false === Locations::has_city($location_country, $location_city)) {
                                throw new Stec_Exception(esc_html__('Invalid location city', 'stec'));
                            }
                        }

                        $new_location = new Location();
                        $new_location->set_address($location_address);
                        $new_location->set_city($location_city);
                        $new_location->set_country($location_country);
                        $new_location->set_coordinates($location_coords);
                        $new_location->set_details($location_details);
                        $new_location->set_visibility('private');

                        if ('' === trim($new_location->get_full_address())) {
                            throw new Stec_Exception(esc_html__('Invalid location', 'stec'));
                        }

                        try {
                            $new_location_id = $new_location->insert_location();
                            $event->set_location($new_location_id);
                        } catch (Stec_Exception $ex) {
                            Admin_Helper::debug_log($ex->getMessage());
                            throw new Stec_Exception(esc_html__('This location is private', 'stec'));
                        }
                    }

                    break;

                default:
                    $location = Admin_Helper::post('location', null, FILTER_VALIDATE_INT);
                    if ($location) {
                        $event->set_location($location);
                    }
            }
        }

        $event->set_custom_meta(array(
            'contact_email' => Admin_Helper::post('contact_email', get_bloginfo('admin_email'), FILTER_VALIDATE_EMAIL)
        ));

        $event->set_review_note(Admin_Helper::post('review_note', ''));

        // Attendance
        // add/edit only if field is available in the builder form
        if (Admin_Helper::isset_post('has_attendance_list')) {
            $attendees       = Admin_Helper::post('attendees', array(), FILTER_DEFAULT, FILTER_REQUIRE_ARRAY);
            $attendees_array = array();

            foreach ($attendees as $attendee) {

                if (!isset($attendee['email']) || false === filter_var($attendee['email'], FILTER_VALIDATE_EMAIL)) {
                    continue;
                }

                $attendee_object = new Event_Meta_Attendee();
                $attendee_object->set_email($attendee['email']);
                $attendee_object->set_mail_sent(isset($attendee['send_email']) ? 0 : 1);
                $attendee_object->set_people($attendee['people']);

                $user = get_user_by('email', $attendee['email']);
                if (false !== $user) {
                    $attendee_object->set_userid($user->ID);
                    $attendee_object->set_name($user->display_name);
                }
                $attendees_array[] = $attendee_object;
            }

            $event->update_attendees($attendees_array);
        }

        // Guests
        // add/edit only if field is available in the builder form
        if (Admin_Helper::isset_post('has_guests_list')) {
            $guests = Admin_Helper::post('guests', array(), FILTER_DEFAULT, FILTER_REQUIRE_ARRAY);
            $event->set_guests(array());
            if ($guests) {

                foreach ($guests as $guest) {

                    if (!isset($guest['name']) || $guest['name'] === '') {
                        continue;
                    }

                    $guest_object = new Event_Meta_Guest();
                    $guest_object->set_name($guest['name']);
                    if (isset($guest['photo'])) {
                        $guest_object->set_photo($guest['photo']);
                    }
                    if (isset($guest['social']) && is_array($guest['social'])) {
                        $guest_object->set_links($guest['social']);
                    }
                    $guest_object->set_about($guest['about']);
                    $event->set_guest($guest_object);
                }
            }
        }

        // Organizers
        if (Admin_Helper::isset_post('organizers')) {
            $organizers = Admin_Helper::post('organizers', array(), FILTER_VALIDATE_INT, FILTER_REQUIRE_ARRAY);

            $event->set_organizers(array());

            if ($organizers) {
                $event->set_organizers($organizers);
            }
        }

        // Schedules
        if (Admin_Helper::isset_post('schedule')) {
            $schedules = Admin_Helper::post('schedule', array(), FILTER_DEFAULT, FILTER_REQUIRE_ARRAY);
            $event->set_schedules(array());
            if ($schedules) {
                foreach ($schedules as $schedule) {

                    $schedule_object = new Event_Meta_Schedule();
                    $schedule_object->set_start_date($schedule['schedule_date_from_alt'] . ' ' . $schedule['schedule_time_hours_from'] . ':' . $schedule['schedule_time_minutes_from'] . ':00');
                    $schedule_object->set_title($schedule['schedule_title']);
                    $schedule_object->set_icon($schedule['schedule_icon']);
                    $schedule_object->set_icon_color($schedule['schedule_icon_color']);
                    $schedule_object->set_details($schedule['schedule_details']);

                    $event->set_schedule($schedule_object);
                }
            }
        }


        // Tickets
        if (Admin_Helper::isset_post('has_tickets') && class_exists('WooCommerce')) {
            $tickets           = Admin_Helper::post('tickets', array(), FILTER_DEFAULT, FILTER_REQUIRE_ARRAY);
            $bookable_products = array();

            if ($tickets) {
                foreach ($tickets as $ticket) {

                    if (isset($ticket['id']) && false !== filter_var($ticket['id'], FILTER_VALIDATE_INT) && $ticket['id'] > 0) {

                        if ('1' !== get_post_meta($ticket['id'], 'stec_bookable', true)) {
                            throw new Stec_Exception(esc_html__('Invalid product id', 'stec'));
                        }

                        // Check if user can edit this ticket
                        if (false === Booking::user_can_manage_ticket($ticket['id'])) {
                            $bookable_product    = new Event_Meta_Bookable_Product();
                            $bookable_product->set_id($ticket['id']);
                            $bookable_products[] = $bookable_product;
                            continue;
                        }

                        $product = new \WC_Product_Stec_Bookable($ticket['id']);
                    } else {
                        $product = new \WC_Product_Stec_Bookable();
                    }

                    $qty           = '';
                    $qty_per_event = '';
                    $price         = '';

                    if (isset($ticket['qty']) && false !== filter_var($ticket['qty'], FILTER_VALIDATE_INT)) {
                        $qty = (int) $ticket['qty'];
                    }

                    if (isset($ticket['qty_per_event']) && false !== filter_var($ticket['qty_per_event'], FILTER_VALIDATE_INT)) {
                        $qty_per_event = $ticket['qty_per_event'];
                    }

                    if (isset($ticket['price']) && false !== is_numeric($ticket['price'])) {
                        $price = $ticket['price'];
                    }

                    $product->set_name($ticket['name']);
                    $product->set_manage_stock(true);
                    $product->set_price($price);
                    $product->set_regular_price($price);
                    $product->set_stock_quantity($qty);
                    $product->set_quantity_limit_per_event($qty_per_event);
                    $product->set_short_description(sanitize_textarea_field($ticket['about']));

                    if (!$qty) {
                        $product->set_manage_stock(false);
                        $product->set_stock_quantity(0);
                        $product->set_stock_status();
                    }

                    if (isset($ticket['wc_book_create_ticket_stop_purchase']) && is_array($ticket['wc_book_create_ticket_stop_purchase'])) {
                        $stop_before       = $ticket['wc_book_create_ticket_stop_purchase'];
                        $stop_before_units = isset($stop_before['units']) ? $stop_before['units'] : 'hours';
                        $stop_before_value = isset($stop_before['value']) ? (int) $stop_before['value'] : 0;

                        switch ($stop_before_units) :
                            case 'days' :
                                $product->set_stop_before($stop_before_value * 24 * 3600);
                                break;

                            case 'hours';
                                $product->set_stop_before($stop_before_value * 3600);
                                break;

                            default:
                                $product->set_stop_before(0);
                        endswitch;
                    }

                    $custom_attributes = $ticket['attr'];
                    $attributes        = array();

                    if (is_array($custom_attributes)) {
                        foreach ($custom_attributes as $attr) {
                            $name  = isset($attr['attr_name']) ? wc_clean($attr['attr_name']) : null;
                            $value = isset($attr['attr_value']) ? wc_clean($attr['attr_value']) : null;

                            if (!$name) {
                                continue;
                            }

                            $attribute = new \WC_Product_Attribute();
                            $attribute->set_name($name);
                            $attribute->set_options(explode('|', $value));
                            $attribute->set_visible(true);

                            $attributes[] = $attribute;
                        }
                    }

                    $product->set_attributes($attributes);

                    $product_id = $product->save();

                    $bookable_product    = new Event_Meta_Bookable_Product();
                    $bookable_product->set_id($product_id);
                    $bookable_products[] = $bookable_product;
                }
            }

            $event->set_bookable_products($bookable_products);
        }

        if (false === $is_edit) {

            $event->set_approved(0);

            if (!$calendar->get_req_approval()) {
                $event->set_approved(1);
            }

            if (!is_user_logged_in()) {
                $cookie = self::get_user_cookie();

                if (!$cookie) {
                    throw new Stec_Exception(esc_html__("You don't have permission to submit events", 'stec'));
                }

                $event->set_custom_meta(array(
                    'created_by_cookie' => $cookie
                ));
            }
        }

        $result = $event->insert_post();

        if (false === is_wp_error($result)) {

            if (is_numeric($result)) {
                $event->set_id($result);

                if ($old_event) {

                    $notify_changes = false;

                    if ($old_event->get_start_date() !== $event->get_start_date()) {
                        $notify_changes = true;
                    }

                    if ($old_event->get_location() !== $event->get_location()) {
                        $notify_changes = true;
                    }

                    if (true === $notify_changes) {
                        Mail::send_event_changed_mail($old_event, $event);
                    }
                }

                return $event;
            }

            throw new Stec_Exception(esc_html__('Unexpected error', 'stec'));
        }

        throw new Stec_Exception($result->get_error_message());
    }

    public static function delete_event($event_id) {

        $event = new Event_Post($event_id);

        if (!$event->get_id()) {
            throw new Stec_Exception(esc_html__('Event does not exist', 'stec'));
        }

        $event->delete_post();

        return true;
    }

    /**
     * Returns minutes list
     * @return array
     */
    public static function get_minutes_array() {
        return Admin_Helper::minutes_array();
    }

    public static function validate_captcha($secret, $response, $ip) {

        $fields = array(
            'secret'   => $secret,
            'response' => $response,
            'remoteip' => $ip
        );

        $fields_string = "";

        foreach ($fields as $key => $value) {
            $fields_string .= $key . '=' . $value . '&';
        }

        rtrim($fields_string, '&');

        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, 'https://www.google.com/recaptcha/api/siteverify');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_POST, count($fields));
        curl_setopt($ch, CURLOPT_POSTFIELDS, $fields_string);

        $result = curl_exec($ch);

        curl_close($ch);

        $json = json_decode($result);

        return isset($json->success) ? (bool) $json->success : false;
    }
  
}
