<?php

namespace Stachethemes\Stec;

use WC_Product_Stec_Bookable;
use WC_Order;
use WC_Order_Item_Product;
use WC;

class Booking {

    protected static $bookable_products = false;
    protected static $filters_added     = false;

    public static function is_stop_before(Event_Post $event, WC_Product_Stec_Bookable $product, $event_offset = 0) {
        // Check stop purchase
        $stop_before      = $product->get_stop_before();
        $now_date         = gmdate('U');
        $event_start_date = $event->get_start_date('U', $event_offset);
        $sales_date       = max(0, $event_start_date - $stop_before);

        return $now_date > $sales_date;
    }

    public static function user_can_manage_ticket($product_id) {

        if ('public' === get_post_meta($product_id, 'stec_admin_visibility', true)) {
            return true;
        } elseif (get_post_field('post_author', $product_id) === get_current_user_id() || is_super_admin()) {
            return true;
        } else {
            return false;
        }
    }

    public static function get_ticket_orders($product_id) {

        if (false === self::user_can_manage_ticket($product_id)) {
            return array();
        }

        if ('1' !== get_post_meta($product_id, 'stec_bookable', true)) {
            throw new Stec_Exception(esc_html__('Invalid ticket id', 'stec'));
        }

        global $wpdb;

        $orders = $wpdb->get_col(
                $wpdb->prepare("
                            SELECT DISTINCT woi.order_id
                            FROM {$wpdb->prefix}woocommerce_order_itemmeta as woim, 
                                 {$wpdb->prefix}woocommerce_order_items as woi, 
                                 {$wpdb->prefix}posts as p
                            WHERE  woi.order_item_id = woim.order_item_id
                            AND woi.order_id = p.ID
                            AND p.post_status IN ( 'wc-completed', 'wc-processing', 'wc-on-hold' )
                            AND woim.meta_key IN ( '_product_id', '_variation_id' )
                            AND woim.meta_value LIKE %d
                            ORDER BY woi.order_item_id DESC",
                        array(
                            $product_id
                        )
        ));

        return $orders;
    }

    public static function get_total_sales($product_id) {
        if ('1' !== get_post_meta($product_id, 'stec_bookable', true)) {
            throw new Stec_Exception(esc_html__('Invalid ticket id', 'stec'));
        }

        $ticket = new WC_Product_Stec_Bookable($product_id);
        return $ticket->get_total_sales();
    }

    public static function get_used_by_count($product_id) {

        $count = 0;

        if ('1' !== get_post_meta($product_id, 'stec_bookable', true)) {
            return $count;
        }

        $events_with_tickets = get_posts(array(
            'post_type'  => 'stec_event',
            'fields'     => 'ids',
            'meta_query' => array(
                array(
                    'key'     => 'bookable_products',
                    'value'   => '',
                    'compare' => '!='
                ))
        ));

        if (!$events_with_tickets) {
            return $count;
        }


        foreach ($events_with_tickets as $event_id) {
            $bookable_products = get_post_meta($event_id, 'bookable_products', true);
            if (is_array($bookable_products)) {
                foreach ($bookable_products as $bookable) {
                    if (false === $bookable instanceof Event_Meta_Bookable_Product) {
                        continue;
                    }

                    if ($bookable->get_id() === $product_id) {
                        $count++;
                    }
                }
            }
        }

        return $count;
    }

    public static function order_has_tickets(Wc_Order $order) {
        $contains_calendar_ticket = false;
        $items                    = $order->get_items();
        foreach ($items as $item) {
            $item_data  = $item->get_data();
            $product_id = filter_var($item_data['product_id'], FILTER_VALIDATE_INT);
            if (!$product_id) {
                continue;
            }
            $product = wc_get_product($product_id);
            if ($product && '1' === $product->get_meta('stec_bookable')) {
                $contains_calendar_ticket = true;
                break;
            }
        }

        return $contains_calendar_ticket;
    }

    private static function add_book_filters_insert_qc_code(WC_Order $order) {
        if (false === self::order_has_tickets($order)) {
            return;
        }

        $order_key                = $order->get_order_key();
        $qr_code_url              = site_url(sprintf('/?task=stec_check_order_key&key=%s', $order_key));
        $qr_code_url_after_filter = apply_filters('stec_ticket_qr_code_url', $qr_code_url, $order);
        $qr_code_img_src          = Admin_Helper::get_qrcode($qr_code_url_after_filter);

        if ($qr_code_img_src) {
            printf('<h2 class="woocommerce-order-details__title">%s</h2>', esc_html__('Calendar QR Code', 'stec'));
            printf('<div>%s</div>',
                    sprintf('<img src="%s" />', $qr_code_img_src)
            );
            printf('<p>%s</p>', esc_html__('Scan this QR Code to check your order status', 'stec'));
        }
    }

    public static function add_book_filters() {

        if (true === self::$filters_added) {
            return;
        }

        add_action('woocommerce_checkout_process', function() {

            $cart              = WC()->cart;
            $has_bookable      = false;
            $contents          = $cart->get_cart_contents();
            $bookable_products = array();

            foreach ($contents as $content) {
                $product = $content['data'];
                if (true === $product instanceof WC_Product_Stec_Bookable) {
                    $has_bookable        = true;
                    $bookable_products[] = $product;
                }
            }

            if (false === $has_bookable) {
                return;
            }

            $items_ids = array_map(function($item) {
                return $item->get_id();
            }, $bookable_products);

            foreach ($bookable_products as $product) {
                $product instanceof WC_Product_Stec_Bookable;
                $requires_ticket = $product->get_meta('stec_requires_ticket');
                $requires_role   = $product->get_meta('stec_requires_role');

                if (is_array($requires_ticket) && $requires_ticket) {
                    foreach ($requires_ticket as $required_ticket_id) {
                        if (false === in_array($required_ticket_id, $items_ids)) {
                            throw new Stec_Exception(sprintf(esc_html__("You can't purchase %s without purchasing %s as well", 'stec'),
                                            $product->get_title(), get_the_title($required_ticket_id)));
                        }
                    }
                }

                if (is_array($requires_role) && $requires_role) {

                    $current_user_roles = Admin_Helper::get_user_roles();

                    if (false === (bool) array_intersect($current_user_roles, $requires_role)) {
                        throw new Stec_Exception(sprintf(esc_html__("You don't have the required user role to purchase %s", 'stec'),
                                        $product->get_title()));
                    }
                }
            }
        });


        add_action('woocommerce_order_details_after_order_table', function(WC_Order $order) {
            self::add_book_filters_insert_qc_code($order);
        });


        add_action('woocommerce_email_after_order_table', function(WC_Order $order) {
            self::add_book_filters_insert_qc_code($order);
        });


        // Add stec variables to cart item
        add_filter('woocommerce_add_cart_item_data', function ( $cartItemData, $productId, $variationId ) {

            $variables = Admin_Helper::post('variables', array(), FILTER_DEFAULT, FILTER_REQUIRE_ARRAY);

            if ($variables) {
                $cartItemData['stec_variables'] = $variables;
            }

            return $cartItemData;
        }, 10, 3);

        // Add stec variables to cart item from session
        add_filter('woocommerce_get_cart_item_from_session', function ( $cartItemData, $cartItemSessionData, $cartItemKey ) {
            if (isset($cartItemSessionData['stec_variables'])) {
                $cartItemData['stec_variables'] = $cartItemSessionData['stec_variables'];
            }

            return $cartItemData;
        }, 10, 3);

        // Cart item meta data
        add_filter('woocommerce_get_item_data', function ( $data, $cartItem ) {

            if (isset($cartItem['stec_variables'])) {

                foreach ($cartItem['stec_variables'] as $var) {

                    $skip = array('event_id', 'event_offset');

                    if (in_array($var['name'], $skip)) {
                        continue;
                    }

                    $data[] = array(
                        'name'  => isset($var['label']) ? $var['label'] : $var['name'],
                        'value' => $var['value']
                    );
                }
            }

            return $data;
        }, 10, 2);

        // Variables as seen in order details
        add_action('woocommerce_add_order_item_meta', function ( $itemId, $values, $key ) {
            if (isset($values['stec_variables'])) {
                foreach ($values['stec_variables'] as $var) {
                    wc_add_order_item_meta($itemId, wc_clean($var['name']), wc_clean($var['value']));
                }
            }
        }, 10, 3);


        // Removes Event Offset from order front-end detail
        add_filter('woocommerce_order_item_get_formatted_meta_data', function($meta, $instance) {

            if (false === ($instance instanceof WC_Order_Item_Product)) {
                return $meta;
            }

            if ('1' !== get_post_meta($instance->get_product_id(), 'stec_bookable', true)) {
                return $meta;
            }

            $filtered_meta = array();

            if (is_array($meta)) {
                foreach ($meta as $var) {
                    $var->display_key = self::get_label($var->key);
                    $filtered_meta[]  = $var;
                }
            }

            return $filtered_meta;
        }, 10, 3);


        // Stec_Bookable filters
        // Add to listing
        add_filter('product_type_selector', function($types) {

            $types['stec_bookable'] = esc_html__('St. Calendar Bookable product', 'stec');
            return $types;
        });

        add_filter('woocommerce_product_class', function ($classname, $product_type) {
            if ($product_type === 'stec_bookable') {
                $classname = 'WC_Product_Stec_Bookable';
            }
            return $classname;
        }, 10, 2);

        // Add tickets tab
        add_filter('woocommerce_product_data_tabs', function($tabs) {

            $tabs['general']['class'][]        = 'show_if_simple show_if_grouped show_if_external show_if_variable';
            $tabs['shipping']['class'][]       = 'hide_if_stec_bookable';
            $tabs['linked_product']['class'][] = 'hide_if_stec_bookable';

            return $tabs;
        });

        // General & Invetory tabs javascript fix
        add_action('admin_footer', function() {

            if ('product' != get_post_type()) {
                return;
            }
            ?>
            <script type="text/javascript">
                (function ($) {
                    $(function () {
                        setTimeout(function () {
                            $('#general_product_data .pricing').addClass('show_if_stec_bookable').show();
                            $('#inventory_product_data ._manage_stock_field').addClass('show_if_stec_bookable').show();
                            $('#inventory_product_data ._sold_individually_field').parent().addClass('show_if_stec_bookable').show();
                            $('#inventory_product_data ._sold_individually_field').addClass('show_if_stec_bookable').show();
                            $('.inventory_options').addClass('show_if_stec_bookable').show();
                            $('.product_data_tabs').children().first().find('a').first().trigger('click');
                            $('.product_data_tabs .general_tab').addClass('show_if_stec_bookable').show();
                        }, 0);
                    });
                })(window.jQuery);
            </script>
            <?php

        });

        // Bookable status listener
        add_action('woocommerce_order_status_changed', function($order_id, $old_status) {
            \Stachethemes\Stec\Booking::set_order_changed($order_id, $old_status);
        }, 10, 3);

        self::$filters_added = true;
    }

    public static function get_bookable_products($inc_all = false) {

        if (self::$bookable_products !== false) {
            return self::$bookable_products;
        }

        $bookable_list = array();

        $query = get_posts(array(
            'post_type'      => array('product'),
            'posts_per_page' => -1,
            'fields'         => 'ids',
            'post_status'    => 'publish',
            'tax_query'      => array(
                array(
                    'taxonomy' => 'product_type',
                    'field'    => 'slug',
                    'terms'    => array('stec_bookable'),
                    'operator' => 'IN'
                )
            ))
        );

        if ($query && is_array($query)) {
            foreach ($query as $id) {
                if (true === self::user_can_manage_ticket($id) && false === $inc_all) {
                    $bookable_list[] = new WC_Product_Stec_Bookable($id);
                }
            }
        }

        self::$bookable_products = $bookable_list;

        return self::$bookable_products;
    }

    public static function get_bookable_products_as_array_list($inc_all = false) {

        $arr = array();

        $bookable = self::get_bookable_products($inc_all);

        foreach ($bookable as $item) {
            $arr[$item->get_id()] = $item->get_title();
        }

        return $arr;
    }

    public static function add_coupon($coupon) {
        WC()->cart->remove_coupons();
        return WC()->cart->add_discount(sanitize_text_field($coupon));
    }

    public static function add_to_cart($item) {

        $id             = (int) $item['id'];
        $quantity       = (int) $item['quantity'];
        $variables      = self::process_item_variables($item);
        $variation      = 0;
        $variations     = array();
        $cart_item_data = array('stec_variables' => $variables);

        /**
         * If valid event
         */
        $event        = self::get_event_from_variables($variables);
        $event_offset = self::get_offset_from_variables($variables);

        if (!$event || !$event->get_id()) {
            throw new Stec_Exception(esc_html__('Event does not exist', 'stec'));
        }

        if (false === $event->is_valid_offset($event_offset)) {
            throw new Stec_Exception(esc_html__('Invalid event date', 'stec'));
        }

        /**
         * If valid product
         */
        $product = wc_get_product($id);

        // Check if exists
        if (!$product || false === $product instanceof WC_Product_Stec_Bookable) {
            throw new Stec_Exception(esc_html__('This item does not exist', 'stec'));
        }

        // Check in stock / is purchasable
        if (false === $product->is_in_stock() || false === $product->is_purchasable()) {
            throw new Stec_Exception(sprintf(esc_html__('%s is out of stock', 'stec'), $product->get_title()));
        }

        if (true === self::is_stop_before($event, $product, $event_offset)) {
            throw new Stec_Exception(esc_html__('Sales for this ticket have ended', 'stec'));
        }

        // Check quantity
        if (false === $product->has_enough_stock($quantity)) {
            throw new Stec_Exception(
                    sprintf(esc_html__('We have only %d in stock for %s', 'stec'),
                            $product->get_stock_quantity(),
                            $product->get_title()
                    )
            );
        }

        // Check event qty is not full
        if (false === self::has_enough_stock_for_event($event, $event_offset, $product, $quantity)) {

            $tickets_left = self::get_tickets_left_for_event($event, $event_offset, $product);

            if ($tickets_left > 0) {
                throw new Stec_Exception(sprintf(esc_html__("We have only %d tickets of %s left for this event date", "stec"), $tickets_left, $product->get_name()));
            } else {
                throw new Stec_Exception(sprintf(esc_html__("All %s tickets are sold out for this event date", "stec"), $product->get_name()));
            }
        }

        if ($product->managing_stock()) {
            $products_qty_in_cart = WC()->cart->get_cart_item_quantities();

            if (isset($products_qty_in_cart[$product->get_stock_managed_by_id()]) && !$product->has_enough_stock($products_qty_in_cart[$product->get_stock_managed_by_id()] + $quantity)) {
                throw new Stec_Exception(
                        sprintf(esc_html__('You cannot add that amount of %1$s to the cart &mdash; we have %2$s in stock and you already have %3$s in your cart.', 'stec'),
                                $product->get_title(),
                                wc_format_stock_quantity_for_display($product->get_stock_quantity(), $product),
                                wc_format_stock_quantity_for_display($products_qty_in_cart[$product->get_stock_managed_by_id()], $product))
                        . sprintf('<br><a href="%s" class="button wc-forward">%s</a>',
                                wc_get_cart_url(),
                                esc_html__('View cart', 'stec'))
                );
            }
        }

        try {
            $added = WC()->cart->add_to_cart($product->get_id(), $quantity, $variation, $variations, $cart_item_data);

            if (false === $added) {
                throw new Stec_Exception(sprintf(esc_html__('Unable to add %s to your cart', 'stec'), $product->get_title()));
            }

            return $added;
        } catch (Exception $ex) {
            throw new Stec_Exception($ex->getMessage());
        }
    }

    public static function get_tickets_left_for_event(Event_Post $event, $event_offset, WC_Product_Stec_Bookable $product) {
        $tickets_sold = self::get_tickets_sold($event, $product->get_id(), $event_offset);

        if (false === is_numeric($tickets_sold)) {
            $tickets_sold = 0;
        }

        // Check if unlimited
        if (!$product->get_quantity_limit_per_event() && !$product->get_stock_quantity()) {
            return 9999;
        }

        $limit = $product->get_quantity_limit_per_event();

        if ($limit <= 0) {
            return $product->get_stock_quantity();
        }

        return max(0, absint($tickets_sold - $limit));
    }

    protected static function has_enough_stock_for_event(Event_Post $event, $event_offset, WC_Product_Stec_Bookable $product, $quantity) {
        $tickets_left = self::get_tickets_left_for_event($event, $event_offset, $product);
        return $tickets_left >= $quantity;
    }

    protected static function get_event_from_variables($variables) {
        foreach ($variables as $var) {
            if ($var['name'] !== 'event_id') {
                continue;
            }

            $id = filter_var($var['value'], FILTER_VALIDATE_INT);

            if (!$id) {
                return null;
            }

            return new Event_Post($id);
        }

        return null;
    }

    protected static function get_offset_from_variables($variables) {
        foreach ($variables as $var) {
            if ($var['name'] !== 'event_offset') {
                continue;
            }

            return (int) $var['value'];
        }

        return 0;
    }

    protected static function process_item_variables($item) {
        $variables = array();

        if ($item['variables'] && is_array($item['variables'])) {
            $variable_index = 0;
            foreach ($item['variables'] as $var_name => $var) {
                $name                                = wc_clean($var_name);
                $value                               = wc_clean($var);
                $variables[$variable_index]['name']  = $name;
                $variables[$variable_index]['value'] = $value;
                $variables[$variable_index]['label'] = self::get_label($name);
                $variable_index++;
            }
        }

        return $variables;
    }

    protected static function get_label($name) {

        $labels = array(
            'event_id'        => esc_html__('Event ID', 'stec'),
            'event_title'     => esc_html__('Event', 'stec'),
            'event_start'     => esc_html__('Start Date', 'stec'),
            'event_end'       => esc_html__('End Date', 'stec'),
            'event_offset'    => esc_html__('Repeat Offset', 'stec'),
            'event_permalink' => esc_html__('Link', 'stec')
        );

        return isset($labels[$name]) ? $labels[$name] : $name;
    }

    /**
     * Revert added tickets count to event meta
     */
    protected static function revert_completed_order(WC_Order $order) {

        $items = $order->get_items();

        foreach ($items as $item) {

            $item_data = $item->get_data();

            $product_id = filter_var($item_data['product_id'], FILTER_VALIDATE_INT);

            if (!$product_id) {
                continue;
            }

            $product = wc_get_product($product_id);

            if (!$product || '1' !== $product->get_meta('stec_bookable')) {
                continue;
            }

            $event_id     = filter_var($item->get_meta('event_id'), FILTER_VALIDATE_INT);
            $event_offset = $item->get_meta('event_offset');
            $quantity     = $item->get_quantity();

            if (!$event_id) {
                continue;
            }

            $event = new Event_Post($event_id);

            if (!$event->get_id()) {
                continue;
            }

            $old_quantity = self::get_tickets_sold($event, $product_id, $event_offset);

            if (false === is_numeric($old_quantity)) {
                $old_quantity = 0;
            }

            $new_quantity = max(0, $old_quantity - $quantity);

            self::set_tickets_sold($event, $new_quantity, $product_id, $event_offset);
        }
    }

    /**
     * Update tickets count to event meta
     */
    protected static function complete_order(WC_Order $order) {

        $items = $order->get_items();
        
        $has_tickets = false;

        foreach ($items as $item) {

            $item_data  = $item->get_data();
            $product_id = filter_var($item_data['product_id'], FILTER_VALIDATE_INT);

            if (!$product_id) {
                continue;
            }

            if ('1' !== get_post_meta($product_id, 'stec_bookable', true)) {
                continue;
            }

            $event_id     = filter_var($item->get_meta('event_id'), FILTER_VALIDATE_INT);
            $event_offset = $item->get_meta('event_offset');
            $quantity     = $item->get_quantity();

            if (!$event_id) {
                continue;
            }

            $event = new Event_Post($event_id);

            if (!$event->get_id()) {
                continue;
            }

            $old_quantity = self::get_tickets_sold($event, $product_id, $event_offset);

            if (false === is_numeric($old_quantity)) {
                $old_quantity = 0;
            }

            $new_quantity = max(0, $old_quantity + $quantity);

            self::set_tickets_sold($event, $new_quantity, $product_id, $event_offset);
            
            $has_tickets = true;

            do_action('stec_after_book_item_order_completed', $item, $event, $event_offset);
        }
        
        if (true === $has_tickets) {
            do_action('stec_after_book_order_completed', $order, $event, $event_offset);
        }
    }

    public static function set_order_changed($order_id, $old_status) {

        $order      = wc_get_order($order_id);
        $new_status = $order->get_status();

        if ($new_status === $old_status) {
            return;
        }

        switch ($new_status) {
            case 'completed' :
                self::complete_order($order);
                break;

            default:
                if ('completed' === $old_status) {
                    self::revert_completed_order($order);
                }
        }
    }

    public static function set_tickets_sold(Event_Post $event, $qty, $product_id, $offset = 0) {

        try {

            if (false === filter_var($event->get_id(), FILTER_VALIDATE_INT)) {
                throw new Stec_Exception('Invalid event id');
            }

            if (false === filter_var($product_id, FILTER_VALIDATE_INT)) {
                throw new Stec_Exception('Invalid product id');
            }

            if (false === filter_var($offset, FILTER_VALIDATE_INT)) {
                throw new Stec_Exception('Invalid offset');
            }

            if (false === $event->is_valid_offset($offset)) {
                throw new Stec_Exception('Invalid offset');
            }

            if (false === filter_var($qty, FILTER_VALIDATE_INT)) {
                throw new Stec_Exception('Invalid qty');
            }

            if ('1' !== get_post_meta($product_id, 'stec_bookable', true)) {
                throw new Stec_Exception('Invalid product type');
            }

            $tickets_sold = self::get_tickets_sold($event, $product_id, $offset);

            if (false === is_array($tickets_sold)) {
                $tickets_sold = array();
            }

            $tickets_sold[$product_id]          = array();
            $tickets_sold[$product_id][$offset] = (int) $qty;

            update_post_meta($event->get_id(), 'stec-bookable-tickets-sold', $tickets_sold);
        } catch (Stec_Exception $ex) {
            Admin_Helper::debug_log('Booking::set_tickets_sold - ' . $ex->getMessage());
        }
    }

    public static function get_tickets_sold(Event_Post $event, $product_id = null, $offset = 0) {
        $tickets_sold = $event->get_custom_meta('stec-bookable-tickets-sold');

        if (!is_array($tickets_sold)) {
            $tickets_sold = array();
        }

        if (!$product_id) {
            return $tickets_sold;
        }

        if (-1 === $offset) {
            return isset($tickets_sold[$product_id]) ? $tickets_sold[$product_id] : 0;
        }

        if (!is_int($product_id) || !is_int($offset)) {
            return 0;
        }

        if (isset($tickets_sold[$product_id]) && isset($tickets_sold[$product_id][$offset])) {
            return $tickets_sold[$product_id][$offset];
        }

        return 0;
    }

}
