<?php

namespace Stachethemes\Stec;

use DateTime;
use DateTimeZone;

class Calendar_Post extends Stec_Post {

    private $post_type       = 'stec_calendar';
    private $post_status     = 'publish';
    private $title           = null;
    private $icon            = null;
    private $color           = '#f15e6e';
    private $timezone        = 'UTC';
    private $writable        = 0;
    private $req_approval    = 1;
    private $back_visibility = 'stec_public';
    private $visibility      = 'stec_public';
    private $author          = null;

    public function __construct($post_id = null) {

        if ((int) $post_id && $this->post_type === get_post_type((int) $post_id)) {
            $this->id              = (int) $post_id;
            $this->post_status     = get_post_status($this->id);
            $this->title           = get_post_field('post_title', $this->id);
            $this->icon            = get_post_meta($this->id, 'icon', true);
            $this->color           = get_post_meta($this->id, 'color', true);
            $this->timezone        = get_post_meta($this->id, 'timezone', true);
            $this->writable        = get_post_meta($this->id, 'writable', true);
            $this->req_approval    = get_post_meta($this->id, 'req_approval', true);
            $this->back_visibility = get_post_meta($this->id, 'back_visibility', true);
            $this->visibility      = get_post_meta($this->id, 'visibility', true);
        }
    }

    public function set_author($author) {
        $this->author = (int) $author;
    }

    public function get_title() {
        return $this->title;
    }

    public function get_icon() {
        return $this->icon;
    }

    public function get_color() {
        return $this->color;
    }

    public function get_timezone() {
        return $this->timezone;
    }

    public function get_writable() {
        return explode(',', $this->writable);
    }

    public function get_req_approval() {
        return (int) $this->req_approval;
    }

    public function get_back_visibility() {
        return explode(',', $this->back_visibility);
    }

    public function get_visibility() {
        return explode(',', $this->visibility);
    }

    public function set_title($title) {
        $this->title = sanitize_text_field($title);
    }

    public function set_icon($icon) {
        $this->icon = sanitize_text_field($icon);
    }

    public function set_color($color) {
        $this->color = sanitize_hex_color($color);
    }

    public function set_timezone($timezone) {
        $this->timezone = sanitize_text_field($timezone);
    }

    public function set_writable($writable) {

        if (is_array($writable)) {
            $writable = implode(',', $writable);
        }
        $this->writable = sanitize_text_field($writable);
    }

    public function set_req_approval($req_approval) {
        $this->req_approval = (int) $req_approval;
    }

    public function set_back_visibility($back_visibility) {

        if (is_array($back_visibility)) {
            $back_visibility = implode(',', $back_visibility);
        }

        $this->back_visibility = sanitize_text_field($back_visibility);
    }

    public function set_visibility($visibility) {

        if (is_array($visibility)) {
            $visibility = implode(',', $visibility);
        }

        $this->visibility = sanitize_text_field($visibility);
    }

    public function get_post_data() {

        return array(
            'ID'           => $this->id,
            'post_author'  => $this->author ? $this->author : get_current_user_id(),
            'post_content' => '',
            'post_title'   => $this->title,
            'post_type'    => $this->post_type,
            'post_status'  => $this->post_status,
            'meta_input'   => array_merge(array(
                'icon'            => $this->icon,
                'color'           => $this->color,
                'timezone'        => $this->timezone,
                'writable'        => $this->writable,
                'req_approval'    => $this->req_approval,
                'back_visibility' => $this->back_visibility,
                'visibility'      => $this->visibility,
                    ), $this->custom_meta)
        );
    }

    public function delete_post() {
        global $wpdb;

        if (false === Calendars::user_can_edit_calendar($this)) {
            throw new Stec_Exception(esc_html__("You don't have permission to delete this calendar", "stec"));
        }

        $result = parent::delete_post();

        if (false === $result) {
            return false;
        }

        // Delete all cron jobs for this calendar
        Cron::delete_calendar_jobs($this->id);

        // Delete all events from this calendar 
        $events = $wpdb->get_results($wpdb->prepare("SELECT POSTS.id FROM {$wpdb->prefix}posts as POSTS
                LEFT JOIN {$wpdb->prefix}postmeta AS META 
                ON META.post_id = POSTS.ID
                WHERE POSTS.post_type = %s AND META.meta_key = %s AND META.meta_value = %d", 'stec_event', 'calid', $this->id));

        $event_ids = array();
        foreach ($events as $event) {
            $event_ids[] = $event->id;
        }

        Events::bulk_delete($event_ids);

        return true;
    }

    public function get_timezone_offset() {

        if (!$this->timezone) {
            return 0;
        }

        $dateTimeZoneUTC     = new DateTimeZone("UTC");
        $dateTimeZoneCAL     = new DateTimeZone($this->timezone);
        $dateTimeUTC         = new DateTime("now", $dateTimeZoneUTC);
        $timezone_utc_offset = $dateTimeZoneCAL->getOffset($dateTimeUTC);

        return $timezone_utc_offset;
    }

    public function get_front_data() {
        return array(
            'id'       => $this->get_id(),
            'title'    => $this->get_title(),
            'color'    => $this->get_color(),
            'timezone' => $this->get_timezone()
        );
    }

    public function insert_post() {

        if ($this->id) {
            if (!Calendars::user_can_edit_calendar($this)) {
                throw new Stec_Exception(esc_html__("You don't have permission to edit this calendar", 'stec'));
            }
        }

        return parent::insert_post();
    }

}
