<?php

namespace Stachethemes\Stec;

class Location {

    private $id;
    private $title;
    private $address;
    private $city;
    private $country;
    private $coordinates;
    private $details;
    private $notes;
    private $taxonomy   = 'stec_event_location';
    private $author;
    private $count;
    private $visibility = 'public';

    public function __construct($id = null) {

        if ((int) $id) {
            $this->id          = (int) $id;
            $term              = get_term($this->id);
            $this->count       = $term->count;
            $this->title       = get_term_meta($this->id, 'title', true);
            $this->address     = get_term_meta($this->id, 'address', true);
            $this->city        = get_term_meta($this->id, 'city', true);
            $this->country     = get_term_meta($this->id, 'country', true);
            $this->coordinates = get_term_meta($this->id, 'coordinates', true);
            $this->details     = get_term_meta($this->id, 'details', true);
            $this->notes       = get_term_meta($this->id, 'notes', true);
            $this->author      = get_term_meta($this->id, 'author', true);
            $this->visibility  = get_term_meta($this->id, 'visibility', true);

            if ($term->name !== $this->get_full_address()) {
                $this->address = $term->name;
            }
        }
    }

    public function get_id() {
        return filter_var($this->id, FILTER_VALIDATE_INT);
    }

    public function get_full_address() {
        $address = array_filter(array(
            $this->get_address(),
            $this->get_city() ? $this->get_city() : null,
            $this->get_country() ? $this->get_country() : null
        ));

        return implode(', ', $address);
    }

    public function get_title() {
        return ($this->title);
    }

    public function get_address() {
        return ($this->address);
    }

    public function get_city() {
        return ($this->city);
    }

    public function get_country() {
        return ($this->country);
    }

    public function get_coordinates() {
        return ($this->coordinates);
    }

    public function get_notes() {
        return ($this->notes);
    }

    public function get_details() {
        return $this->details;
    }

    public function set_title($title) {
        $this->title = sanitize_text_field($title);
    }

    public function set_address($address) {
        $this->address = sanitize_text_field($address);
    }

    public function set_city($city) {
        $this->city = sanitize_text_field($city);
    }

    public function set_country($country) {
        $this->country = sanitize_text_field($country);
    }

    public function set_coordinates($coordinates) {
        $this->coordinates = sanitize_text_field($coordinates);
    }

    public function set_details($details) {
        $this->details = sanitize_textarea_field($details);
    }

    public function set_notes($notes) {
        $this->notes = sanitize_textarea_field($notes);
    }

    public function get_data() {
        return (object) array(
                    'id'           => $this->get_id(),
                    'full_address' => $this->get_full_address(),
                    'title'        => $this->get_title(),
                    'address'      => $this->get_address(),
                    'city'         => $this->get_city(),
                    'country'      => $this->get_country(),
                    'coordinates'  => $this->get_coordinates(),
                    'details'      => $this->get_details()
        );
    }

    public function get_events_count() {
        if ($this->get_id()) {
            return (int) $this->count;
        }

        return 0;
    }

    public function insert_location() {

        if ($this->id) {
            $term = wp_update_term($this->id, $this->taxonomy, array(
                'name' => $this->get_full_address()
            ));
        } else {
            $term = wp_insert_term($this->get_full_address(), $this->taxonomy);
        }

        if (is_wp_error($term)) {

            if ('term_exists' === $term->get_error_code() && false !== filter_var($term->get_error_data(), FILTER_VALIDATE_INT)) {
                $this->id = (int) $term->get_error_data();
            } else {
                throw new Stec_Exception($term->get_error_message());
            }
        }

        if (!$this->id) {
            if (!isset($term['term_id'])) {
                throw new Stec_Exception('term_id not found');
            }
            $this->id = $term['term_id'];
        }

        update_term_meta($this->id, 'coordinates', $this->coordinates);
        update_term_meta($this->id, 'title', $this->title);
        update_term_meta($this->id, 'address', $this->address);
        update_term_meta($this->id, 'city', $this->city);
        update_term_meta($this->id, 'country', $this->country);
        update_term_meta($this->id, 'details', $this->details);
        update_term_meta($this->id, 'notes', $this->notes);
        update_term_meta($this->id, 'author', get_current_user_id());
        update_term_meta($this->id, 'visibility', $this->visibility);

        return $this->id;
    }

    public function get_visibility() {
        return ($this->visibility);
    }

    public function set_visibility($visibility) {
        $this->visibility = sanitize_text_field($visibility);
    }

    public function delete() {
        if ($this->get_author() && $this->get_author() === get_current_user_id() || is_super_admin()) {
            return wp_delete_term($this->id, $this->taxonomy);
        }
    }

    public function get_author() {
        return (int) $this->author;
    }

}
