<?php

namespace Stachethemes\Stec;

class Organizer {

    private $id;
    private $name;
    private $photo;
    private $details;
    private $email;
    private $social     = array();
    private $notes;
    private $taxonomy   = 'stec_event_organizer';
    private $author;
    private $count;
    private $visibility = 'public';
    private $permalink  = null;

    public function __construct($id = null) {

        if (filter_var($id, FILTER_VALIDATE_INT)) {
            $this->id = (int) $id;
            $term     = get_term($this->id);

            if (!is_wp_error($term) && $term) {
                $this->count      = $term->count;
                $this->name       = Admin_Helper::decrypt(get_term_meta($this->id, 'name', true));
                $this->photo      = Admin_Helper::decrypt(get_term_meta($this->id, 'photo', true));
                $this->details    = Admin_Helper::decrypt(get_term_meta($this->id, 'details', true));
                $this->email      = Admin_Helper::decrypt(get_term_meta($this->id, 'email', true));
                $this->social     = maybe_unserialize(Admin_Helper::decrypt(get_term_meta($this->id, 'social', true)));
                $this->notes      = Admin_Helper::decrypt(get_term_meta($this->id, 'notes', true));
                $this->author     = Admin_Helper::decrypt(get_term_meta($this->id, 'author', true));
                $this->visibility = get_term_meta($this->id, 'visibility', true);
                $this->permalink  = $this->get_permalink();
            } else {
                $term->id = null;
            }
        }
    }

    public function get_id() {
        return $this->id;
    }

    public function get_permalink() {

        if (null !== $this->permalink) {
            return esc_url($this->permalink);
        }

        $page_id = Admin_Helper::get_stec_page('organizer');

        if (!$page_id) {
            return null;
        }

        $page = Admin_Helper::get_permalinks('organizer', 'stec_organizer');

        if (defined('ICL_LANGUAGE_CODE')) {
            $default_locale = Admin_Helper::get_default_locale();
            if ($default_locale !== ICL_LANGUAGE_CODE) {
                $page = ICL_LANGUAGE_CODE . '-' . $page;
            }
        }

        $name = $this->get_name();

        return esc_url(home_url($page . '/' . $name));
    }

    public function get_email() {
        return $this->email;
    }

    public function set_email($email) {

        if (!$email) {
            $this->email = '';
        } else {
            if (false !== filter_var($email, FILTER_VALIDATE_EMAIL)) {
                $this->email = $email;
            }
        }
    }

    public function get_photo() {
        return filter_var($this->photo, FILTER_VALIDATE_INT, FILTER_NULL_ON_FAILURE);
    }

    public function get_photo_src($size = 'full') {
        
        if (!$this->get_photo()) {
            return null;
        }
        
        $photo = wp_get_attachment_image_src($this->get_photo(), $size);
        return esc_url($photo[0]);
    }

    public function get_details() {
        return Admin_Helper::esc($this->details, 'noscript');
    }

    public function get_social() {

        $filtered = array();

        foreach ($this->social as $social) {
            if ($social['link']) {

                $social['ico'] = ($social['ico']);

                if (false !== filter_var($social['link'], FILTER_VALIDATE_EMAIL)) {
                    // do nothing
                } elseif (false !== filter_var($social['link'], FILTER_VALIDATE_URL)) {
                    $social['link'] = ($social['link']);
                } else {
                    $social['link'] = ($social['link']);
                }

                $filtered[] = $social;
            }
        }

        return $filtered;
    }

    public function get_name() {
        return $this->name;
    }

    public function get_notes() {
        return $this->notes;
    }

    public function set_name($name) {
        $this->name = sanitize_text_field($name);
    }

    public function set_photo($photo) {
        $this->photo = filter_var($photo, FILTER_VALIDATE_INT, FILTER_NULL_ON_FAILURE);
    }

    public function set_details($details) {
        $this->details = sanitize_textarea_field($details);
    }

    public function set_social(Array $social_links) {

        $filtered_social_links = array();

        foreach ($social_links as $social) {

            if (!isset($social['link']) || !isset($social['ico'])) {
                continue;
            }

            $value = sanitize_text_field($social['link']);

            if ($value === '') {
                continue;
            }

            $filtered_social_links[] = array(
                'ico'  => sanitize_text_field($social['ico']),
                'link' => $value
            );
        }

        $this->social = $filtered_social_links;
    }

    public function set_notes($notes) {
        $this->notes = sanitize_textarea_field($notes);
    }

    public function get_data() {
        return (object) array(
                    'id'        => $this->get_id(),
                    'name'      => $this->get_name(),
                    'email'     => $this->get_email(),
                    'photo'     => $this->get_photo(),
                    'photo_src' => $this->get_photo_src(),
                    'details'   => $this->get_details(),
                    'social'    => $this->get_social(),
                    'permalink' => $this->get_permalink()
        );
    }

    public function get_events_count() {
        if ($this->get_id()) {
            return $this->count;
        }

        return 0;
    }

    public function insert_organizer() {

        $term = null;

        // Check if email is already registered somewhere
        if ($this->get_email()) {
            $terms = get_terms($this->taxonomy, array(
                'hide_empty' => false,
                'fields'     => 'ids',
                'meta_query' => array(
                    array(
                        'key'     => 'email',
                        'value'   => Admin_Helper::encrypt($this->get_email()),
                        'compare' => '='
                    )
                )
            ));

            if ($terms) {
                foreach ($terms as $term_id) {
                    if ((int) $term_id !== (int) $this->get_id()) {
                        throw new Stec_Exception(sprintf(esc_html__('E-mail %s is already in use by another organizer', 'stec'), $this->get_email()));
                    }
                }
            }
        }

        // If new organizer
        if (!$this->get_id()) {

            $term = wp_insert_term(microtime(), $this->taxonomy);

            if (is_wp_error($term)) {
                throw new Stec_Exception($term->get_error_message());
            }

            if (!isset($term['term_id'])) {
                throw new Stec_Exception('term_id not found');
            }

            $this->id = $term['term_id'];
        }

        update_term_meta($this->id, 'name', $this->name ? Admin_Helper::encrypt($this->name) : '');
        update_term_meta($this->id, 'photo', $this->photo ? Admin_Helper::encrypt($this->photo) : '');
        update_term_meta($this->id, 'details', $this->details ? Admin_Helper::encrypt($this->details) : '');
        update_term_meta($this->id, 'email', $this->email ? Admin_Helper::encrypt($this->email) : '');
        update_term_meta($this->id, 'social', $this->social ? Admin_Helper::encrypt($this->social) : array());
        update_term_meta($this->id, 'notes', $this->notes ? Admin_Helper::encrypt($this->notes) : '');
        update_term_meta($this->id, 'author', get_current_user_id() ? Admin_Helper::encrypt(get_current_user_id()) : '');
        update_term_meta($this->id, 'visibility', $this->visibility);

        return $this->id;
    }

    public function delete() {
        if ($this->author && $this->author === get_current_user_id() || is_super_admin()) {
            return wp_delete_term($this->id, $this->taxonomy);
        }
    }

    public function get_author() {
        return (int) $this->author;
    }

    function get_visibility() {
        return $this->visibility;
    }

    function set_visibility($visibility) {
        $this->visibility = sanitize_text_field($visibility);
    }

}
